rs = require("resolution_solution")
-- Refer ro source code of library, for rs.init() to get full list of avaliable options or their explanation.
-- but in most cases you only need to specify game width/height and default scale mode.
rs.init({width = 640, height = 480, mode = 3})
-- This function allow you to change color of bars that you will appear in aspect and pixel perfect modes.
-- By default, they will have black color, but you can change it and even make transparent.
-- To change individual color, use rs.r, rs.g, rs.b, rs.a for red, green, blue and alpha.
-- Also rs.getColor() will return 4 arguments with currect colors and rs.defaultColor() will return default black color.
rs.setColor(0.1, 0.5, 0.2, 0.5)

-- Filter, works best for pixeleted raphics.
-- can be use without arguments, which same as "true".
-- It's simple wrapper for love.graphics.setDefaultFilter().
-- Refer source code for more info.
rs.nearestFilter(true)

-- Make window resizeable. I strongly suggest you to always make window resiable, via this love function or conf.lua
-- After all, this library was designed for this.
rs.setMode(800, 600, {resizable = true})
-- Show library name and version in title.
love.window.setTitle(tostring(rs._NAME .. " v." .. rs._VERSION))

-- Example rectangle, that demonstrate how you can implement, for example, mouse collision detection,
-- and other translate functions.
local rectangle1 = {
  x = 100,
  y = 100,
  w = 100,
  h = 100,
  click = 0
}

local image = love.graphics.newImage("test.png")

-- Show/hide rectangle around scaled area. 
local showGameZone = true

-- library was designed to update at love.resize() (it possible to update at love.update(), but it's not something that you want to do),
-- so place it there. Also, side not: never forget to use rs.init() (even if you don't need to change any settings) at least 1 at start of game/scene.
-- This is required since until 1st window resize, library will be not updated, so no scale, no offset, nothing will be calculated.
love.resize = function(w, h)
  rs.resize(w, h)
end

-- Change options with keyborad
love.keypressed = function(key, scancode, isrepeat)
  if key == "f1" then
    rs.switchScaleMode()
  elseif key == "f2" then
      rs.switchBars()
  elseif key == "f3" then
      rs.switchDebug()
  elseif key == "f4" then
      showGameZone = not showGameZone
  elseif key == "f5" then
      rs.switchPixelHack()
  end
end

-- Example of how you can implement mouse collision detection function.
local mouseFunc = function(x, y, w, h)

  -- Translate mouse to ingame coordinates
  local mx, my = rs.toGame(love.mouse.getPosition())
  if mx  >= x                 and -- left
     my    >= y                 and -- top
     mx    <= x         +    w  and -- right
     my    <= y         +    h  then -- bottom
     return true
    end

    return false
end

love.mousepressed = function(x, y, button, istouch, presses)

    -- Example of usage for mouse collision.
    -- Add 1 to counter if clicked.
    if rs.isMouseInside() and mouseFunc(rectangle1.x, rectangle1.y, rectangle1.w, rectangle1.h) and button == 1 then
      rectangle1.click = rectangle1.click + 1
    end

    -- Example of how to use and transka scaled coordinates to screen coordinates.
    -- Set mouse cursor to rectangle.
    if button == 2 then
      love.mouse.setPosition(rs.toScreenX(rectangle1.x), rs.toScreenY(rectangle1.y))
    end

    -- Another translation example.
    -- Move rectangle to cursor.
    if button == 3 then
      rectangle1.x, rectangle1.y = rs.toGame(love.mouse.getPosition())
    end
end

love.draw = function()
  -- Start scaling
  rs.start()
  -- Background color.
    love.graphics.setBackgroundColor(0, 0.4, 0.6, 1)

    love.graphics.draw(image)

    -- Change rectangle color if we touch it.
    if rs.isMouseInside() and mouseFunc(rectangle1.x, rectangle1.y, rectangle1.w, rectangle1.h) then
      love.graphics.setColor(1, 0.5, 0.5, 1)
    else
        love.graphics.setColor(0.5, 0.5, 0.5, 1)
    end

    -- Draw rectangle.
    love.graphics.rectangle("fill", rectangle1.x, rectangle1.y, rectangle1.w, rectangle1.h)

    -- Show counter and explanation.
    love.graphics.setColor(1, 1, 1, 1)
    love.graphics.print("Click on me!" .. tostring(rectangle1.click) ..  "\nYou can't click on me, if i behind\nbars, because library\ncan take care of it!", rectangle1.x, rectangle1.y)

    -- Scaled text.
    love.graphics.print("I'm scaled text!", 200, 50)

    -- Example of how you can implement UI that should be scaled separately/differently from game.
    rs.unscaleStart()
      love.graphics.setColor(1, 1, 1, 1)
      love.graphics.print("I'm unscaled, despite being in-between rs.start() and rs.stop()!\nAlso bars draws ontop of me!", 180, 50)
    rs.unscaleStop()

  -- Stop scaling.
  rs.stop()

  -- Bars text.
  love.graphics.setColor(1, 1, 1, 1)
  love.graphics.print("Bars can be any color you want! Not only black!", rs.windowWidth - 200, 0)

  love.graphics.setColor(0, 0, 0, 1)
    -- Example of how you can use rs.gameZone.
    -- Draw rectangle.
  if showGameZone then
    -- love.graphics.rectangle("line", rs.gameZone.x, rs.gameZone.y, rs.gameZone.w, rs.gameZone.h)
  end

  -- Call debug function.
  rs.debugFunc()

  -- Instructions.
  love.graphics.setColor(1, 1, 1, 1)
  love.graphics.print("Press F1 to change scaleMode. F2 to enable/disable bars. F3 to enable/disable debug info. Press f4 to show/hide game zone borders.\nPress F5 to activate/deactivate pixel perfect hack (make sure to be in scaling mode 3). When active, try resize window and see if you notice difference. \nTry to change window size and click on rectangle. Press right mouse button to move cursor to rectangle. Press middle mouse to move rectangle under cursor.", 0, rs.windowHeight - 100)
end